@@ -324,7 +324,7 @@ class LayerSet(BaseObject):
 
     def _fontSaveWasCompleted(self):
         """
-        When saving a UFOZ, the underlying ZipFS object is closed.
+        On saving a UFO, the underlying fs object is closed.
         The objects then stored in layer._glyphSet contain references
         to a closed, and therefore unusable, filesystem. To remedy this,
         after the save is completed this method will be called and new
@@ -333,10 +333,22 @@ class LayerSet(BaseObject):
         font = self.font
         font.close()
         font._reader = reader = UFOReader(font.path, validate=font.ufoLibReadValidate)
-        if reader.fileStructure is UFOFileStructure.ZIP:
-            for layerName in self.layerOrder:
-                layer = self[layerName]
-                layer._glyphSet = reader.getGlyphSet(layerName=layerName, validateRead=self.ufoLibReadValidate)
+        defaultLayerName = self._defaultLayerName
+        for layerName in self.layerOrder:
+            layer = self[layerName]
+            if reader.formatVersionTuple < (3, 0):
+                # ufo2 or less has not layers
+                # map only to the default layer to the ufo2 glyphSet
+                # set all all other layers internal _glyphSet object to None
+                if defaultLayerName == layerName:
+                    glyphSet = reader.getGlyphSet(validateRead=self.ufoLibReadValidate)
+                else:
+                    glyphSet = None
+            else:
+                # we do have layers and the must be available after a save action
+                glyphSet = reader.getGlyphSet(layerName=layerName, validateRead=self.ufoLibReadValidate)
+
+            layer._glyphSet = glyphSet
 
     # ------------------------
     # Notification Observation
