@@ -1,12 +1,13 @@
-from __future__ import print_function
-from collections import OrderedDict
 """
 A flexible and relatively robust implementation
 of the Observer Pattern.
 """
 
+from __future__ import print_function
+from collections import OrderedDict
 import sys
 import weakref
+from fnmatch import fnmatchcase
 
 """
 
@@ -20,7 +21,7 @@ registry : {
         (notification, observable) : OrderedDict(
             observer : method name
         )
-    }
+}
 
 holds : {
     (notification, observable, observer) : {
@@ -35,6 +36,12 @@ disabled : {
     (notification, observable, observer) : count
 }
 
+identifierRegistry : {
+        (notification, observable) : OrderedDict(
+            observer : identifier
+        )
+}
+
 """
 
 
@@ -42,14 +49,16 @@ class NotificationCenter(object):
 
     def __init__(self):
         self._registry = {}
+        self._observerKeyBacktrack = {}
         self._holds = {}
         self._disabled = {}
+        self._identifierRegistry = {}
 
     # -----
     # Basic
     # -----
 
-    def addObserver(self, observer, methodName, notification=None, observable=None):
+    def addObserver(self, observer, methodName, notification=None, observable=None, identifier=None):
         """
         Add an observer to this notification dispatcher.
 
@@ -62,15 +71,20 @@ class NotificationCenter(object):
         * **observable** The object to observe. If this is None,
           all notifications with the name provided as *notification*
           will be posted to the *observer*.
+        * **identifier** None or a string identifying the observation.
+          There is no requirement that the string be unique. A reverse
+          domain naming scheme is recommended, but there are no
+          requirements for the structure of the string.
 
         If None is given for both *notification* and *observable*
-        **all** notifications posted will be sent to the method
-        given method of the observer.
+        **all** notifications posted will be sent to the given method
+        of the observer.
 
         The method that will be called as a result of the action
         must accept a single *notification* argument. This will
         be a :class:`Notification` object.
         """
+        assert notification != "all", "'all' is a reserved name and can not be used as a notification."
         if observable is not None:
             observable = weakref.ref(observable)
         observer = weakref.ref(observer)
@@ -82,6 +96,15 @@ class NotificationCenter(object):
                 notification=key[0], observable=key[1](), observer=observer(), method1=self._registry[key][observer], method2=methodName
             )
         self._registry[key][observer] = methodName
+        if observer not in self._observerKeyBacktrack:
+            self._observerKeyBacktrack[observer] = {}
+        if observable not in self._observerKeyBacktrack[observer]:
+            self._observerKeyBacktrack[observer][observable] = set()
+        self._observerKeyBacktrack[observer][observable].add(key)
+        if identifier is not None:
+            if key not in self._identifierRegistry:
+                self._identifierRegistry[key] = OrderedDict()
+            self._identifierRegistry[key][observer] = identifier
 
     def hasObserver(self, observer, notification, observable):
         """
@@ -103,19 +126,39 @@ class NotificationCenter(object):
 
         * **observer** A registered object.
         * **notification** The notification that the observer was registered
-          to be notified of.
+          to be notified of. If this is `"all"`, all notifications for
+          the *observable* will be removed for *observer*.
         * **observable** The object being observed.
         """
         if observable is not None:
             observable = weakref.ref(observable)
-        key = (notification, observable)
-        if key not in self._registry:
-            return
         observer = weakref.ref(observer)
-        if observer in self._registry[key]:
-            del self._registry[key][observer]
-        if not len(self._registry[key]):
-            del self._registry[key]
+        if notification == "all":
+            keys = set(self._observerKeyBacktrack[observer][observable])
+        else:
+            keys = [
+                (notification, observable)
+            ]
+        for key in keys:
+            if key not in self._registry:
+                continue
+            if observer in self._registry[key]:
+                del self._registry[key][observer]
+            if not len(self._registry[key]):
+                del self._registry[key]
+            if observer in self._observerKeyBacktrack:
+                if observable in self._observerKeyBacktrack[observer]:
+                    if key in self._observerKeyBacktrack[observer][observable]:
+                        self._observerKeyBacktrack[observer][observable].remove(key)
+                    if not self._observerKeyBacktrack[observer][observable]:
+                        del self._observerKeyBacktrack[observer][observable]
+                if not self._observerKeyBacktrack[observer]:
+                    del self._observerKeyBacktrack[observer]
+            if key in self._identifierRegistry:
+                if observer in self._identifierRegistry[key]:
+                    del self._identifierRegistry[key][observer]
+                if not len(self._identifierRegistry[key]):
+                    del self._identifierRegistry[key]
 
     def postNotification(self, notification, observable, data=None):
         assert notification is not None
@@ -366,6 +409,63 @@ class NotificationCenter(object):
         key = (notification, observable, observer)
         return key in self._disabled
 
+    # ----
+    # Find
+    # ----
+
+    def findObservations(self, observer=None, notification=None, observable=None, identifier=None):
+        """
+        Find observations matching the given arguments based
+        on the values that were passed during addObserver.
+        A value of None for any of these indicates that all
+        should be considered to match the value. In the case
+        of identifier, strings will be matched using fnmatch.fnmatchcase.
+        The returned value will be a list of dictionaries with
+        this format:
+
+            [
+                {
+                    observer=<...>
+                    observable=<...>
+                    methodName="..."
+                    notification="..."
+                    identifier="..."
+                }
+            ]
+        """
+        if observer is not None:
+            observer = weakref.ref(observer)
+        if observable is not None:
+            observable = weakref.ref(observable)
+        found = []
+        for (otherNotification, otherObservable), observerDict in self._registry.items():
+            if notification is not None:
+                if otherNotification != notification:
+                    continue
+            if observable is not None:
+                if otherObservable != observable:
+                    continue
+            for otherObserver, methodName in observerDict.items():
+                otherIdentifier = None
+                if (otherNotification, otherObservable) in self._identifierRegistry:
+                    otherIdentifier = self._identifierRegistry[otherNotification, otherObservable].get(otherObserver)
+                if otherIdentifier is None and identifier is not None:
+                    continue
+                if otherIdentifier is not None and identifier is not None:
+                    if not fnmatchcase(otherIdentifier, identifier):
+                        continue
+                if observer is not None:
+                    if otherObserver != observer:
+                        continue
+                observation = dict(
+                    observer=otherObserver(),
+                    observable=otherObservable(),
+                    notification=otherNotification,
+                    identifier=otherIdentifier
+                )
+                found.append(observation)
+        return found
+
 
 class Notification(object):
 
